# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0011_auto_20150419_1006'),
    ]

    operations = [
        migrations.CreateModel(
            name='Style',
            fields=[
                ('label', models.CharField(default=b'', help_text='Optional label for this style plugin.', max_length=128, verbose_name='label', blank=True)),
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('class_name', models.CharField(default=b'info', max_length=50, verbose_name='class name', blank=True, choices=[(b'info', b'info'), (b'new', b'new'), (b'hint', b'hint')])),
                ('id_name', models.CharField(default=b'', max_length=50, verbose_name='id name', blank=True)),
                ('tag_type', models.CharField(default=b'div', max_length=50, verbose_name='tag Type', choices=[(b'div', 'div'), (b'article', 'article'), (b'section', 'section'), (b'span', 'inline')])),
                ('padding_left', models.SmallIntegerField(null=True, verbose_name='padding left', blank=True)),
                ('padding_right', models.SmallIntegerField(null=True, verbose_name='padding right', blank=True)),
                ('padding_top', models.SmallIntegerField(null=True, verbose_name='padding top', blank=True)),
                ('padding_bottom', models.SmallIntegerField(null=True, verbose_name='padding bottom', blank=True)),
                ('margin_left', models.SmallIntegerField(null=True, verbose_name='margin left', blank=True)),
                ('margin_right', models.SmallIntegerField(null=True, verbose_name='margin right', blank=True)),
                ('margin_top', models.SmallIntegerField(null=True, verbose_name='margin top', blank=True)),
                ('margin_bottom', models.SmallIntegerField(null=True, verbose_name='margin bottom', blank=True)),
                ('additional_class_names', models.TextField(help_text='Comma separated list of additional classes to apply to tag_type', verbose_name='additional classes', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def forwards(self, orm):
        for obj in orm.Style.objects.all():
            obj.txt_additional_class_names = obj.additional_class_names
            obj.save()

    def backwards(self, orm):
        for obj in orm.Style.objects.all():
            obj.additional_class_names = obj.txt_additional_class_names[0:200]
            obj.save()

    models = {
        u'aldryn_style.style': {
            'Meta': {'object_name': 'Style', '_ormbases': ['cms.CMSPlugin']},
            'additional_class_names': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'class_name': ('django.db.models.fields.CharField', [], {'default': "'container'", 'max_length': '50', 'blank': 'True'}),
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'+'", 'unique': 'True', 'primary_key': 'True', 'to': "orm['cms.CMSPlugin']"}),
            'id_name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '50', 'blank': 'True'}),
            'margin_bottom': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'margin_left': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'margin_right': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'margin_top': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'padding_bottom': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'padding_left': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'padding_right': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'padding_top': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'tag_type': ('django.db.models.fields.CharField', [], {'default': "'div'", 'max_length': '50'}),
            'txt_additional_class_names': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'cms.cmsplugin': {
            # NOTE: We've commented out a couple of fields below. This will
            # allow this data migration to actually work under 3.0 and 3.1
            # (where we switched to Treebeard). Manual testing shows that the
            # migration still works fine. Please do not un-comment these lines.
            # Further note that all of this will disappear once we stop
            # supporting Django 1.6.
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            # 'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            # 'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.CMSPlugin']", 'null': 'True', 'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            # 'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            # 'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        }
    }

    complete_apps = ['aldryn_style']
    symmetrical = True



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import re

import warnings

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.utils.six import string_types

from cms.models import CMSPlugin


CLASS_NAMES = getattr(
    settings,
    'ALDRYN_STYLE_CLASS_NAMES',
    (
        ('container', _('container')),
        ('content', _('content')),
        ('teaser', _('teaser')),
    )
)

CLASS_NAME_FORMAT = re.compile(r'^\w[\w_-]*$')
TAG_TYPE_FORMAT = re.compile(r'\w[\w\d]*$')


def get_html_tag_types():
    tag_types = getattr(settings, 'ALDRYN_STYLE_ALLOWED_TAGS', None)
    # If this is a backwards-compatible list of String tag names, then we
    # should check them for sanity, otherwise, we'll assume Aldryn has
    # already done this.
    if tag_types:
        # Remove anything that doesn't look like an HTML tag
        for tag_item in tag_types:
            if isinstance(tag_item, string_types):
                tag = tag_item.strip()
            else:
                # This is probably a list of tuples a la:
                #     ( ..., ('option', 'option', ), ...)
                try:
                    tag = tag_item[0]
                except TypeError:
                    # OK, not sure what this is...
                    continue
            if not TAG_TYPE_FORMAT.match(tag):
                warnings.warn(_('ALDRYN STYLE: "{0}" was omitted from '
                                'ALDRYN_STYLE_ALLOWED_TAGS as it does '
                                'not look like a valid HTML tag.').format(tag))
                tag_types.remove(tag_item)

    # Could be that it was initially empty, or, none of the supplied entries
    # looked right, in either of these cases, use the default set as defined
    # in version 1.0.1.
    if not tag_types:
        tag_types = [
            'div', 'article', 'section', 'p', 'span',
            'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
            'header', 'footer',
        ]

    return tuple([(tag, tag) for tag in tag_types])


@python_2_unicode_compatible
class Style(CMSPlugin):
    """
    A CSS Style Plugin
    """
    DIV_TAG = 'div'
    HTML_TAG_TYPES = get_html_tag_types()

    label = models.CharField(
        _('label'), max_length=128, default='', blank=True,
        help_text=_('Optional label for this style plugin.'))
    cmsplugin_ptr = models.OneToOneField(
        CMSPlugin, related_name='+', parent_link=True)
    class_name = models.CharField(
        _('class name'), choices=CLASS_NAMES, default=CLASS_NAMES[0][0],
        max_length=50, blank=True)
    id_name = models.CharField(
        _('id name'), max_length=50, blank=True, default='')
    tag_type = models.CharField(
        verbose_name=_('tag Type'), max_length=50, choices=HTML_TAG_TYPES,
        default=DIV_TAG)

    padding_top = models.SmallIntegerField(
        _('padding top'), blank=True, null=True)
    padding_right = models.SmallIntegerField(
        _('padding right'), blank=True, null=True)
    padding_bottom = models.SmallIntegerField(
        _('padding bottom'), blank=True, null=True)
    padding_left = models.SmallIntegerField(
        _('padding left'), blank=True, null=True)

    margin_top = models.SmallIntegerField(
        _('margin top'), blank=True, null=True)
    margin_right = models.SmallIntegerField(
        _('margin right'), blank=True, null=True)
    margin_bottom = models.SmallIntegerField(
        _('margin bottom'), blank=True, null=True)
    margin_left = models.SmallIntegerField(
        _('margin left'), blank=True, null=True)

    additional_class_names = models.TextField(
        verbose_name=_('additional classes'),
        blank=True,
        help_text=_('Comma separated list of additional classes to apply to '
                    'tag_type')
    )

    def __str__(self):
        display = self.tag_type or ''
        if self.additional_class_names:
            display = '{0} ({1})'.format(display, self.additional_class_names)
        if self.label:
            display = '“{0}”: {1}'.format(self.label, display)
        return display

    def clean(self):
        if self.additional_class_names:
            additional_class_names = list(
                html_class.strip() for html_class in
                self.additional_class_names.split(','))
            for class_name in additional_class_names:
                class_name = class_name.strip()
                if not CLASS_NAME_FORMAT.match(class_name):
                    raise ValidationError(
                        _('"{0}" is not a proper css class name.').format(
                            class_name)
                    )
            self.additional_class_names = ', '.join(
                set(additional_class_names))

    @property
    def get_additional_class_names(self):
        if self.additional_class_names:
            # Removes any extra spaces
            return ' '.join((
                html_class.strip() for html_class in
                self.additional_class_names.split(',')))
        return ''

    @property
    def inline_style(self):
        styles = []
        if self.padding_top:
            styles.append('padding-top: {0:d}px;'.format(self.padding_top))
        if self.padding_right:
            styles.append('padding-right: {0:d}px;'.format(self.padding_right))
        if self.padding_bottom:
            styles.append('padding-bottom: {0:d}px;'.format(self.padding_bottom))
        if self.padding_left:
            styles.append('padding-left: {0:d}px;'.format(self.padding_left))
        if self.margin_top:
            styles.append('margin-top: {0:d}px;'.format(self.margin_top))
        if self.margin_right:
            styles.append('margin-right: {0:d}px;'.format(self.margin_right))
        if self.margin_bottom:
            styles.append('margin-bottom: {0:d}px;'.format(self.margin_bottom))
        if self.margin_left:
            styles.append('margin-left: {0:d}px;'.format(self.margin_left))
        return ' '.join(styles)
